@@ -391,34 +391,20 @@ chat_send (EmpathyChat  *chat,
 
 	chat_sent_message_add (chat, msg);
 
-	if (g_str_has_prefix (msg, "/clear")) {
+	if (strcmp (msg, "/clear") == 0) {
 		empathy_chat_view_clear (chat->view);
 		return;
 	}
 
-	/* Blacklist messages begining by '/', except for "/me" and "/say"
-	 * because they are handled in EmpathyMessage */
-	if (msg[0] == '/' &&
-	    !g_str_has_prefix (msg, "/me") &&
-	    !g_str_has_prefix (msg, "/say")) {
-		/* Also allow messages with two slashes before the first space,
-		 * so it is possible to send an /unix/path */
-		int slash_count = 0, i;
-		for (i = 0; msg[i] && msg[i] != ' ' && slash_count < 2; i++) {
-			if (msg[i] == '/')
-				slash_count++;
-		}
-		if (slash_count == 1) {
-			empathy_chat_view_append_event (chat->view,
-				_("Unsupported command"));
-			return;
-		}
-	}
+	message = empathy_message_new_from_entry (msg);
 
-	/* We can send the message */
-	message = empathy_message_new (msg);
-	empathy_tp_chat_send (priv->tp_chat, message);
-	g_object_unref (message);
+	if (message == NULL) {
+		empathy_chat_view_append_event (chat->view,
+			_("Unsupported command"));
+	} else {
+		empathy_tp_chat_send (priv->tp_chat, message);
+		g_object_unref (message);
+	}
 }
 
 static void
@@ -537,7 +523,7 @@ chat_message_received_cb (EmpathyTpChat  *tp_chat,
 
 static void
 chat_send_error_cb (EmpathyTpChat          *tp_chat,
-		    EmpathyMessage         *message,
+		    const gchar            *message_body,
 		    TpChannelTextSendError  error_code,
 		    EmpathyChat            *chat)
 {
@@ -566,7 +552,7 @@ chat_send_error_cb (EmpathyTpChat          *tp_chat,
 	}
 
 	str = g_strdup_printf (_("Error sending message '%s': %s"),
-			       empathy_message_get_body (message),
+			       message_body,
 			       error);
 	empathy_chat_view_append_event (chat->view, str);
 	g_free (str);
