@@ -99,9 +99,19 @@ builder_get_file_valist (const gchar *filename,
 
 	gui = gtk_builder_new ();
 	if (!gtk_builder_add_from_file (gui, filename, &error)) {
-		g_critical ("GtkBuilder Error: %s", error->message);
+		g_critical ("GtkBuilder Error (%s): %s",
+				filename, error->message);
 		g_clear_error (&error);
 		g_object_unref (gui);
+
+		/* we need to iterate and set all of the pointers to NULL */
+		for (name = first_object; name;
+		     name = va_arg (args, const gchar *)) {
+			object_ptr = va_arg (args, GObject**);
+
+			*object_ptr = NULL;
+		}
+
 		return NULL;
 	}
 
@@ -1321,6 +1331,31 @@ empathy_get_toplevel_window (GtkWidget *widget)
 	return NULL;
 }
 
+/** empathy_make_absolute_url_len:
+ * @url: an url
+ * @len: a length
+ *
+ * Same as #empathy_make_absolute_url but for a limited string length
+ */
+gchar *
+empathy_make_absolute_url_len (const gchar *url,
+			       guint len)
+{
+	g_return_val_if_fail (url != NULL, NULL);
+
+	if (g_str_has_prefix (url, "ghelp:") ||
+	    g_str_has_prefix (url, "mailto:") ||
+	    strstr (url, ":/")) {
+		return g_strndup (url, len);
+	}
+
+	if (strstr (url, "@")) {
+		return g_strdup_printf ("mailto:%.*s", len, url);
+	}
+
+	return g_strdup_printf ("http://%.*s", len, url);
+}
+
 /** empathy_make_absolute_url:
  * @url: an url
  *
@@ -1335,19 +1370,7 @@ empathy_get_toplevel_window (GtkWidget *widget)
 gchar *
 empathy_make_absolute_url (const gchar *url)
 {
-	g_return_val_if_fail (url != NULL, NULL);
-
-	if (g_str_has_prefix (url, "ghelp:") ||
-	    g_str_has_prefix (url, "mailto:") ||
-	    strstr (url, ":/")) {
-		return g_strdup (url);
-	}
-
-	if (strstr (url, "@")) {
-		return g_strdup_printf ("mailto:%s", url);
-	}
-
-	return g_strdup_printf ("http://%s", url);
+	return empathy_make_absolute_url_len (url, strlen (url));
 }
 
 void
